@@ -8,7 +8,7 @@
 # References:   1) http://search.cpan.org/dist/Image-MetaData-JPEG/
 #               2) http://www.ozhiker.com/electronics/pjmt/jpeg_info/meta.html
 #               3) http://www.cybercom.net/~dcoffin/dcraw/
-#               4) Iliah Borg private communication (LibRaw)
+#               IB) Iliah Borg private communication (LibRaw)
 #
 # Notes:        There really isn't much public information about Kodak formats.
 #               The only source I could find was Image::MetaData::JPEG, which
@@ -24,7 +24,7 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.37';
+$VERSION = '1.41';
 
 sub ProcessKodakIFD($$$);
 sub ProcessKodakText($$$);
@@ -1360,6 +1360,10 @@ my %sceneModeUsed = (
             TagTable => 'Image::ExifTool::Kodak::TextualInfo',
         },
     },
+    # 0x03f2 - FlashMode (ref IB)
+    # 0x03f3 - FlashCompensation (ref IB)
+    # 0x03f8 - MinAperture (ref IB)
+    # 0x03f9 - MaxAperture (ref IB)
     0x03fc => { #3
         Name => 'WhiteBalance',
         Writable => 'int16u',
@@ -1378,25 +1382,36 @@ my %sceneModeUsed = (
         Groups => { 2 => 'Time' },
         Writable => 'string',
     },
-    0x0406 => { #4
+    0x0406 => { #IB
         Name => 'CameraTemperature',
         # (when count is 2, values seem related to temperature, but are not Celius)
         Condition => '$count == 1',
+        Groups => { 2 => 'Camera' },
         Writable => 'rational64s',
         PrintConv => '"$val C"',
         PrintConvInv => '$val=~s/ ?C//; $val',
     },
+    0x0407 => { #IB
+        Name => 'AdapterVoltage',
+        Groups => { 2 => 'Camera' },
+        Writable => 'rational64u',
+    },
+    0x0408 => { #IB
+        Name => 'BatteryVoltage',
+        Groups => { 2 => 'Camera' },
+        Writable => 'rational64u',
+    },
     0x0414 => { Name => 'NCDFileInfo',      Writable => 'string' },
     0x0846 => { #3
         Name => 'ColorTemperature',
         Writable => 'int16u',
     },
-    0x0848 => 'WB_RGBLevelsDaylight', #4
-    0x0849 => 'WB_RGBLevelsTungsten', #4
-    0x084a => 'WB_RGBLevelsFluorescent', #4
-    0x084b => 'WB_RGBLevelsFlash', #4
-    0x084c => 'WB_RGBLevelsCustom', #4
-    0x084d => 'WB_RGBLevelsAuto', #4
+    0x0848 => 'WB_RGBLevelsDaylight', #IB
+    0x0849 => 'WB_RGBLevelsTungsten', #IB
+    0x084a => 'WB_RGBLevelsFluorescent', #IB
+    0x084b => 'WB_RGBLevelsFlash', #IB
+    0x084c => 'WB_RGBLevelsCustom', #IB
+    0x084d => 'WB_RGBLevelsAuto', #IB
     0x0852 => 'WB_RGBMul0', #3
     0x0853 => 'WB_RGBMul1', #3
     0x0854 => 'WB_RGBMul2', #3
@@ -1405,26 +1420,26 @@ my %sceneModeUsed = (
     0x085d => { Name => 'WB_RGBCoeffs1', Binary => 1 }, #3
     0x085e => { Name => 'WB_RGBCoeffs2', Binary => 1 }, #3
     0x085f => { Name => 'WB_RGBCoeffs3', Binary => 1 }, #3
-    0x0903 => { Name => 'BaseISO', Writable => 'rational64u' }, #4 (ISO before digital gain)
+    # 0x089d => true analogue ISO values possible (ref IB)
+    # 0x089e => true analogue ISO used at capture (ref IB)
+    # 0x089f => ISO calibration gain (ref IB)
+    # 0x08a0 => ISO calibration gain table (ref IB)
+    # 0x08a1 => exposure headroom coefficient (ref IB)
+    0x0903 => { Name => 'BaseISO', Writable => 'rational64u' }, #IB (ISO before digital gain)
     # 0x090d: linear table (ref 3)
-    0x09ce => { Name => 'SensorSerialNumber', Writable => 'string', Groups => { 2 => 'Camera' } }, #4
+    0x09ce => { Name => 'SensorSerialNumber', Writable => 'string', Groups => { 2 => 'Camera' } }, #IB
     # 0x0c81: some sort of date (manufacture date?) - PH
-    # 0x089d => true analogue ISO values possible (ref 4)
-    # 0x089e => true analogue ISO used at capture (ref 4)
-    # 0x089f => ISO calibration gain (ref 4)
-    # 0x08a0 => ISO calibration gain table (ref 4)
-    # 0x08a1 => exposure headroom coefficient (ref 4)
-    0x0ce5 => { Name => 'FirmwareVersion',  Writable => 'string' },
-    0x0e4c => { #4
+    0x0ce5 => { Name => 'FirmwareVersion',  Writable => 'string', Groups => { 2 => 'Camera' } },
+    0x0e4c => { #IB
         Name => 'KodakLook',
         Format => 'undef',
         Writable => 'string',
         ValueConv => '$val=~tr/\0/\n/; $val',
         ValueConvInv => '$val=~tr/\n/\0/; $val',
     },
-    0x1389 => { Name => 'InputProfile',     Writable => 'undef', Binary => 1 }, #4
-    0x138a => { Name => 'KodakLookProfile', Writable => 'undef', Binary => 1 }, #4
-    0x138b => { Name => 'OutputProfile',    Writable => 'undef', Binary => 1 }, #4
+    0x1389 => { Name => 'InputProfile',     Writable => 'undef', Binary => 1 }, #IB
+    0x138a => { Name => 'KodakLookProfile', Writable => 'undef', Binary => 1 }, #IB
+    0x138b => { Name => 'OutputProfile',    Writable => 'undef', Binary => 1 }, #IB
     # 0x1390: value: "DCSProSLRn" (tone curve name?) - PH
     0x1391 => { Name => 'ToneCurveFileName',Writable => 'string' },
     0x1784 => { Name => 'ISO',              Writable => 'int32u' }, #3
@@ -1576,14 +1591,14 @@ my %sceneModeUsed = (
     0xc353 => {
         Name => 'CameraOwner',
         Writable => 'undef',
-        RawConv => 'Image::ExifTool::Exif::ConvertExifText($self,$val)',
+        RawConv => 'Image::ExifTool::Exif::ConvertExifText($self,$val,$tag)',
         RawConvInv => 'Image::ExifTool::Exif::EncodeExifText($self,$val)',
     },
     0xc354 => {
         Name => 'SerialNumber',
         Writable => 'undef',
         Groups => { 2 => 'Camera' },
-        RawConv => 'Image::ExifTool::Exif::ConvertExifText($self,$val)', #PH
+        RawConv => 'Image::ExifTool::Exif::ConvertExifText($self,$val,$tag)', #PH
         RawConvInv => 'Image::ExifTool::Exif::EncodeExifText($self,$val)',
     },
     0xc355 => 'UserSelectGroupTitle',
@@ -1648,7 +1663,7 @@ my %sceneModeUsed = (
     0 => 'DigitalEffectsVersion',
     1 => {
         Name => 'DigitalEffectsName',
-        PrintConv => 'Image::ExifTool::Exif::ConvertExifText($self,$val)',
+        PrintConv => 'Image::ExifTool::Exif::ConvertExifText($self,$val,"DigitalEffectsName")',
     },
     2 => 'DigitalEffectsType',
 );
@@ -1660,7 +1675,7 @@ my %sceneModeUsed = (
     0 => 'BordersVersion',
     1 => {
         Name => 'BorderName',
-        PrintConv => 'Image::ExifTool::Exif::ConvertExifText($self,$val)',
+        PrintConv => 'Image::ExifTool::Exif::ConvertExifText($self,$val,"BorderName")',
     },
     2 => 'BorderID',
     3 => 'BorderLocation',
@@ -1839,6 +1854,15 @@ my %sceneModeUsed = (
     },
 );
 
+# tags in "frea" atom of Kodak PixPro SP360 MP4 videos (ref PH)
+%Image::ExifTool::Kodak::frea = (
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
+    NOTES => 'Information stored in the "frea" atom of Kodak PixPro SP360 MP4 videos.',
+    # tima - 4 bytes: "0 0 0 0x20" or "0 0 0 0x0a"
+    thma => { Name => 'ThumbnailImage', Groups => { 2 => 'Preview' }, Binary => 1 },
+    scra => { Name => 'PreviewImage',   Groups => { 2 => 'Preview' }, Binary => 1 },
+);
+
 # preview information in free/Scrn atom of MP4 videos (ref PH)
 %Image::ExifTool::Kodak::Scrn = (
     GROUPS => { 0 => 'MakerNotes', 2 => 'Image' },
@@ -1849,6 +1873,7 @@ my %sceneModeUsed = (
     2 => { Name => 'PreviewImageLength', Format => 'int32u' },
     4 => {
         Name => 'PreviewImage',
+        Groups => { 2 => 'Preview' },
         Format => 'undef[$val{2}]',
         RawConv => '$self->ValidateImage(\$val, $tag)',
     },
@@ -2044,7 +2069,7 @@ interpret Kodak maker notes EXIF meta information.
 
 =head1 AUTHOR
 
-Copyright 2003-2014, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2017, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
